package com.sillycycle.bagleyd.tertris;

/*
 * @(#)Tertris.java
 *
 * Copyright 1993 - 2020  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.JFrame;

/**
 * The <code>Tertris</code> class extends the
 * <code>TertrisCanvas</code> for handling the responses to the user
 * for the <code>TertrisFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

class Tertris extends TertrisCanvas implements Runnable, MouseWheelListener,
		MouseListener, KeyListener {
	private static final long serialVersionUID = 42L;

	private volatile boolean paused = false;
	private volatile boolean fall;
	private volatile boolean resetTime = true;
	static final int[] speeds = {
		100, 92, 84, 75, 68, 60, 52, 45, 39, 32,
		26, 21, 16, 12, 8, 5, 3, 2, 1, 0
	};
	int nextFall = 100 * speeds[level - 1], now = 0;
	int newRows = 0;
	private volatile boolean quit = false;
	private transient Thread gameRunner;

	public Tertris(JFrame frame, int level, int columns, int fill,
			boolean fixedFill, boolean suddenAppear, boolean bonus,
			boolean cw, boolean grid, boolean showNext,
			Color bg) {
		this.frame = frame;
		this.level = level;
		this.columns = columns;
		this.rows = 4 * columns / 3;
		this.fill = fill;
		this.fixedFill = fixedFill;
		this.suddenAppear = suddenAppear;
		this.bonus = bonus;
		this.cw = cw;
		this.grid = grid;
		this.showNext = showNext;
		this.background = bg;
		setBackground(bg);
		score = 0;
		completedRows = 0;
		fall = true;
		firstPaint = true;
		if (frame != null) {
			this.addKeyListener(this);
			this.addMouseWheelListener(this);
			this.addMouseListener(this);
			setCursor(new Cursor(Cursor.HAND_CURSOR));
		}
		start();
	}

	void gameOver(boolean fast) {
		banner(TertrisInterface.MSG_END);
		if (!paused)
			fillLines();
		if (invert)
			invertGrid();
		addScore();
		//showScores(TertrisInterface.SHOWSCORES);
		if (!fast) {
			try {
				Thread.sleep(3000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		newThing();
		if (showNext) {
			clearNext(frame);
			drawNext(frame);
		}
		score = 0;
		completedRows = 0;
		clearField();
		fall = true;
		sizeInit();
		initFill();
		drawField();
	}

	boolean moveOne(int move) {
		int lines;

		if ((move == TertrisInterface.DROP) ||
				((move == TertrisInterface.FALL) &&
				atBottom())) {
			fall = false;
			tryMove(move);
			putUnit();
			lines = checkLines();
			if (dropPaused) {
				dropPaused = false;
			} else {
				score += (fill + ((grid) ? 0 : 1) + level +
				  lines + ((suddenAppear) ? 1 : 0)) *
				  ((curSpec.bonus) ? 1 : 3) *
				  ((showNext) ? 1 : 3) /
				  (1 + ((curSpec.mixed) ? 1 : 0));
				score += ((showNext) ? 3 : 5) *
				  level * level * lines * lines *
				  ((curSpec.bonus) ? 3 : 5) * startlevel *
				  (curSpec.units + ((curSpec.diagonal) ? 1 : 0)) *
				  (curSpec.units + ((curSpec.diagonal) ? 1 : 0)) /
				  (16 * (1 + ((curSpec.mixed) ? 1 : 0)));
			}
			completedRows += lines;
			newRows += lines;
			nextSpec.bonus = false;
			if ((newRows > TertrisInterface.threshold(level)) &&
			    (level < TertrisInterface.MAX_LEVELS)) {
				level++;
				((TertrisFrame) frame).setLevel(level);
				newRows = 0;
				if (bonus && TertrisInterface.bonusLegal(nextSpec.units, bonus))
					nextSpec.bonus = true;
			}
			drawStatus();
			newThing();
			if (showNext) {
				clearNext(frame);
				drawNext(frame);
			}
			if (overlapping()) {
				gameOver(false);
				return false;
			}
			repaint();
			fall = true;
			return true;
		}
		tryMove(move);
		nextSpec.bonus = false;
		if ((newRows > TertrisInterface.threshold(level)) &&
			  (level < TertrisInterface.MAX_LEVELS)) {
			level++;
			((TertrisFrame) frame).setLevel(level);
			newRows = 0;
			if (bonus && TertrisInterface.bonusLegal(nextSpec.units, bonus))
				nextSpec.bonus = true;
			drawStatus();
			if (showNext) {
				clearNext(frame);
				drawNext(frame);
			}
		}
		return false;
	}

	void addScore() {
		((TertrisFrame) frame).writeScore(score);
	}

	void quitGame() {
		banner(TertrisInterface.MSG_END);
		fillLines();
		addScore();
		quit = true;
	}

	void playing() {
//		drawTitle();
//		drawStatus();
//		drawField();
		drawFrame(!paused);
		((TertrisFrame) frame).updateScore(completedRows, score);
		if (resetTime) {
			nextFall = 100 * speeds[level - 1];
		} else {
			now += 10 * speeds[level - 1];
		}
		if (now > nextFall) {
			processAdvance();
			resetTime = true;
			now = 0;
		} else {
			resetTime = processAdvance();
		}
	}

	boolean processAdvance() {
		boolean gotNew = false;

		if (fall && !paused)
			gotNew = moveOne(TertrisInterface.FALL);
		return gotNew;
	}

	public boolean processKey(char key) {
		boolean gotNew = false;
		char charPressed = key;

		if (!fall)
			return false; /* fixes threading problems */
		switch (charPressed) {
		case '>':
		case '.':
			if (!paused && level < TertrisInterface.MAX_LEVELS) {
				level++;
				((TertrisFrame) frame).setLevel(level);
				newRows = 0;
				drawStatus();
			}
			break;
// Cheat
		case '<':
		case ',':
			if (!paused && level > TertrisInterface.MIN_LEVELS) {
				level--;
				newRows = 0;
				((TertrisFrame) frame).setLevel(level);
				nextSpec.bonus = false;
				drawStatus();
			}
			break;
		case '@':
			((TertrisFrame) frame).toggleSound();
			break;
		case '?':
			((TertrisFrame) frame).descriptionHelp();
			break;
		case '!':
			((TertrisFrame) frame).featuresHelp();
			break;
		case '^':
			((TertrisFrame) frame).referencesHelp();
			break;
		case 'a':
		case 'A':
			((TertrisFrame) frame).aboutHelp();
			break;
		case 'j':
		case 'J':
		case 's':
		case 'S':
			if (!paused)
				gotNew = moveOne(TertrisInterface.MOVE_LEFT) || gotNew;
			break;
		case 'k':
		case 'K':
		case 'd':
		case 'D':
			if (!paused)
				gotNew = moveOne(TertrisInterface.ROTATE) || gotNew;
			break;
		case 'l':
		case 'L':
		case 'f':
		case 'F':
			if (!paused)
				gotNew = moveOne(TertrisInterface.MOVE_RIGHT) || gotNew;
			break;
		case ' ':
		case '\r':
		case '\n':
			if (!paused)
				gotNew = moveOne(TertrisInterface.DROP) || gotNew;
			break;
		case '/':
		case 'i':
		case 'I':
		case 'e':
		case 'E':
			if (!paused)
				gotNew = moveOne(TertrisInterface.REFLECT) || gotNew;
			break;
		case 'c':
		case 'C':
			gameOver(true);
			break;
		case '+':
		case '=':
			if (!paused) {
				nextSpec.units++;
				redoNext();
				if (showNext) {
					clearNext(frame);
					drawNext(frame);
				}
				((TertrisFrame) frame).setUnits(nextSpec.units);
				newRows = 0;
	//			repaint();
			}
			break;
		case '-':
		case '_':
			if (!paused) {
				nextSpec.units--;
				nextSpec.bonus = false;
				redoNext();
				if (showNext) {
					clearNext(frame);
					drawNext(frame);
				}
				newRows = 0;
				((TertrisFrame) frame).setUnits(nextSpec.units);
	//			repaint();
			}
			break;
		case ']':
		case '}':
			columns++;
			((TertrisFrame) frame).setColumns(columns);
			break;
		case '[':
		case '{':
			if (columns > PolyiamondThing.MAX_UNITS) {
				columns--;
				((TertrisFrame) frame).setColumns(columns);
			}
			break;
		case '0':
			this.fill = 10;
			((TertrisFrame) frame).setFill(fill);
			break;
		case '9':
		case '8':
		case '7':
		case '6':
		case '5':
		case '4':
		case '3':
		case '2':
		case '1':
			this.fill = charPressed - '0';
			((TertrisFrame) frame).setFill(fill);
			break;
		case '`':
			this.fill = 0;
			((TertrisFrame) frame).setFill(fill);
			break;
		case 'x':
		case 'X':
			fixedFill = !fixedFill;
			((TertrisFrame) frame).setFixedFill(fixedFill);
			break;
		case 'r':
		case 'R':
			suddenAppear = !suddenAppear;
			((TertrisFrame) frame).setSuddenAppear(suddenAppear);
			break;
		case 'g':
		case 'G':
			if (!paused) {
				nextSpec.diagonal = !nextSpec.diagonal;
				redoNext();
				if (showNext) {
					clearNext(frame);
					drawNext(frame);
				}
				((TertrisFrame) frame).setDiagonal(
					nextSpec.diagonal);
				newRows = 0;
	//			repaint();
			}
			break;
		case 'm':
		case 'M':
			if (!paused) {
				nextSpec.mixed = !nextSpec.mixed;
				nextSpec.mixed = nextSpec.mixed &&
					TertrisInterface.mixedLegal(nextSpec.units, nextSpec.mixed);
				redoNext();
				if (showNext) {
					clearNext(frame);
					drawNext(frame);
				}
				((TertrisFrame) frame).setMixed(
					nextSpec.mixed);
				newRows = 0;
	//			repaint();
			}
			break;
		case 'b':
		case 'B':
			bonus = !bonus;
			((TertrisFrame) frame).setBonus(bonus);
			break;
		case 'w':
		case 'W':
			cw = !cw;
			((TertrisFrame) frame).setClockwise(cw);
			break;
		case 'p':
		case 'P':
			((TertrisFrame) frame).setPlainPiece(!getPlainPiece());
			break;
		case 'n':
		case 'N':
			if (!paused) {
				showNext = !showNext;
				((TertrisFrame) frame).setShowNext(showNext);
				if (showNext) {
					drawNext(frame);
				} else {
					clearNext(frame);
				}
	//			repaint();
			}
			break;
		case '#':
			if (!paused) {
				grid = !grid;
				drawGrid();
			}
			break;
		case '|':
		case '\\':
			if (paused) {
				paused = false;
				banner(null);
				if (showNext) {
					clearNext(frame);
					drawNext(frame);
				}
			} else {
				pause();
			}
			break;
		case '%':
			((TertrisFrame) frame).stopGame();
			break;
		case '&':
			((TertrisFrame) frame).clearGame();
			break;
		case '~':
			((TertrisFrame) frame).setupGame();
			break;
		case 'q':
		case 'Q':
		case '\03': /* Control-C */
			quitGame();
			break;
		default:
			/*XBell(display, 0);
			repaint();
			break;*/
		}
		/* if (paused) {
			XNextEvent(display, &ev);
			flag = true;
			gotNew = true;	*//* for resetting time *//*
			gettimeofday(&oldsample, &tzone);
		} else
			flag = false; */
		return gotNew;
	}

	void redrawAll() {
		drawTitle();
		drawStatus();
		redrawPaint = true;
		repaint();
		if (showNext)
			drawNext(frame);
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		focus = true;
		framePaint = true;
		paused = false;
		banner(null);
		if (showNext) {
			clearNext(frame);
			drawNext(frame);
		}
	}

	@Override
	public void mouseExited(MouseEvent event) {
		pause();
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		int modifiers = event.getModifiersEx();
		char charPressed = (char) 0;

		if ((modifiers & InputEvent.BUTTON1_DOWN_MASK) ==
				InputEvent.BUTTON1_DOWN_MASK)
			charPressed = 'j';
		else if ((modifiers & InputEvent.BUTTON2_DOWN_MASK) ==
				InputEvent.BUTTON2_DOWN_MASK)
			charPressed = 'k';
		else if ((modifiers & InputEvent.BUTTON3_DOWN_MASK) ==
				InputEvent.BUTTON3_DOWN_MASK)
			charPressed = 'l';
		resetTime = processKey(charPressed);
	}

	@Override
	public void mousePressed(MouseEvent event) {
		/* Method required but mousePressed does work */
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		/* Method required but mousePressed does work */
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();

		resetTime = processKey(((notches < 0) ? 'i' : ' '));
	}

	/* void showScores(int num) {
		int i = 0;
		GameScore curs;

		System.err.println("            TERTRIS  HALL OF FAME");
		System.err.println();
		System.err.println(
			"   # USER            SCORE   L    R  HOST         DATE");
		while (read(fd, (char *) &curs, SCORESIZE) == SCORESIZE) {
			i++;
			if ((num == 0) || (i <= num))
				System.err.println(
					"%4d %-12s%9s %3s %4s  %-12s %-s",
					i, curs.myname, curs.score, curs.level,
					curs.rows, curs.myhost, curs.mydate);
		}
		if (i == 1)
			System.err.println("There is 1 score to date.");
		else
			System.err.println("There are " + i +
				" scores to date.");
	}*/

	public final void start() {
		if (gameRunner == null) {
			gameRunner = new Thread(this);
			gameRunner.start();
		}
	}

	public void stop() {
		/*if (gameRunner.isAlive())
			gameRunner.stop();*/
		gameRunner = null;
	}

	public void pause() {
		paused = true;
		dropPaused = true;
		drawFrame(false);
		banner(TertrisInterface.MSG_PAUSED);
	}

	@Override
	public void run() {
		inits();
		sizeInit();
		specInit(TertrisFrame.getUnits(),
			TertrisFrame.getDiagonal(),
			TertrisFrame.getMixed());
		initFill();
		repaint();
		while (!quit) {
			try {
				playing();
				if (Thread.interrupted()) {
					throw new InterruptedException();
				}
				Thread.sleep(10 * speeds[level - 1]);
			} catch (InterruptedException e) {
				//e.printStackTrace();
			}
		}
		if ((TertrisFrame) frame != null)
			((TertrisFrame) frame).quit();
	}

	@Override
	public void keyPressed(KeyEvent event) {
		char charPressed = KeyEvent.CHAR_UNDEFINED;

		switch (event.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			((TertrisFrame) frame).shuffleDown();
			return;
		case KeyEvent.VK_NUMPAD0:
		case KeyEvent.VK_NUMPAD1:
		case KeyEvent.VK_NUMPAD3:
		case KeyEvent.VK_NUMPAD7:
		case KeyEvent.VK_NUMPAD9:
			break;
		case KeyEvent.VK_F11:
		case KeyEvent.VK_DIVIDE:
			charPressed = 'i';
			break;
		case KeyEvent.VK_UP:
		case KeyEvent.VK_KP_UP:
		case KeyEvent.VK_NUMPAD8:
			charPressed = 'k';
			break;
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_KP_LEFT:
		case KeyEvent.VK_NUMPAD4:
			charPressed = 'j';
			break;
		case KeyEvent.VK_F12:
		case KeyEvent.VK_CLEAR:
		case KeyEvent.VK_NUMPAD5:
			charPressed = 'k';
			break;
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_KP_RIGHT:
		case KeyEvent.VK_NUMPAD6:
			charPressed = 'l';
			break;
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_KP_DOWN:
		case KeyEvent.VK_NUMPAD2:
			charPressed = ' ';
			break;
		default:
			charPressed = event.getKeyChar();
		}
		resetTime = processKey(charPressed);
	}

	@Override
	public void keyReleased(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void keyTyped(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	public void setPause(boolean paused) {
		this.paused = paused;
	}

	/* Synchronized causing deadlock */
	public /*synchronized*/ void setLevel(int level) {
		this.level = level;
		newRows = 0;
		drawStatus();
	}

	public void setUnits(int units) {
		nextSpec.units = units;
	}

	public void setFixedFill(boolean fixedFill) {
		this.fixedFill = fixedFill;
	}

	public void setSuddenAppear(boolean suddenAppear) {
		this.suddenAppear = suddenAppear;
		redoNext();
	}

	public void setDiagonal(boolean diagonal) {
		nextSpec.diagonal = diagonal;
		newRows = 0;
	}

	public void setMixed(boolean mixed) {
		nextSpec.mixed = mixed;
		newRows = 0;
	}

	public void setBonus(boolean bonus) {
		this.bonus = bonus;
	}

	public void setClockwise(boolean cw) {
		this.cw = cw;
	}

	public void setGrid(boolean grid) {
		this.grid = grid;
		drawGrid();
	}

	public void setShowNext(boolean showNext) {
		this.showNext = showNext;
		if (showNext) {
			drawNext(frame);
		} else {
			clearNext(frame);
		}
	}

	public void setColumns(int columns) {
		this.columns = columns;
		this.rows = 4 * columns / 3;
		score = 0;
		completedRows = 0;
		clearField();
		sizeInit();
		inits();
		drawField();
		if (showNext) {
			clearNext(frame);
			drawNext(frame);
		}
	}

	public int getColumns() {
		return this.columns;
	}

	public void setFill(int fill) {
		this.fill = fill;
		score = 0;
		completedRows = 0;
		clearField();
		sizeInit();
		inits();
		drawField();
		if (showNext) {
			clearNext(frame);
			drawNext(frame);
		}
	}

	public int getFill() {
		return this.fill;
	}
}

/**
 * The <code>GameScore</code> class holds the high scores.
 */

class GameScore {
	String myname, myhost, mydate;
	String score;
	String level;
	String rows;
}
