package com.sillycycle.bagleyd.tertris;

/*
 * @(#)TertrisDialog.java
 *
 * Structure from Java Sourcebook by Ed Anuff, pp 287-289
 *
 * Copyright 1993 - 2018  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.Spinner;

/**
 * The <code>TertrisDialog</code> class holds the configuration
 * setup for the <code>TertrisFrame</code>.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class TertrisDialog extends Dialog {
	private static final long serialVersionUID = 42L;
	static boolean active = false;
	static TertrisFrame frame = null;

	static volatile int unitsValue = PolyiamondThing.DEFAULT_UNITS;
	static int columnsValue = TertrisInterface.DEFAULT_COLUMNS;
	static int fillValue = 4;
	static boolean fixedFillValue = true;
	static boolean suddenAppearValue = true;
	static volatile boolean diagonalValue = false;
	static volatile boolean mixedValue = false;
	static boolean bonusValue = true;
	static boolean clockwiseValue = false;
	static boolean plainPieceValue = false;
	static boolean showNextValue = true;
	static boolean gridValue = true;
	static boolean soundValue = false;

	Spinner unitsChanger = new Spinner("Polyiamond Size",
		unitsValue, PolyiamondThing.MIN_UNITS,
		PolyiamondThing.MAX_UNITS);
	Spinner columnsChanger = new Spinner("Field Size",
		columnsValue, TertrisInterface.MIN_COLUMNS,
		TertrisInterface.MAX_COLUMNS);
	Spinner fillChanger = new Spinner("Fill",
		fillValue, 0,
		TertrisInterface.MAX_FILL);
	/*Slider unitsChanger = new Slider("Polyiamond Size",
		unitsValue, PolyiamondThing.MIN_UNITS,
		PolyiamondThing.MAX_UNITS);
	Slider columnsChanger = new Slider("Field Size",
		columnsValue, TertrisInterface.MIN_COLUMNS,
		TertrisInterface.MAX_COLUMNS);
	Slider fillChanger = new Slider("Fill",
		fillValue, 0,
		TertrisInterface.MAX_FILL);*/
	JCheckBox fixedFillCheckBox = new JCheckBox("Fixed Fill");
	JCheckBox suddenAppearCheckBox = new JCheckBox("Sudden Appear");
	JCheckBox diagonalCheckBox = new JCheckBox("Diagonal");
	JCheckBox mixedCheckBox = new JCheckBox("Mixed");
	JCheckBox bonusCheckBox = new JCheckBox("Bonus");
	JCheckBox clockwiseCheckBox = new JCheckBox("Clockwise");
	JCheckBox plainPieceCheckBox = new JCheckBox("Plain Piece");
	JCheckBox showNextCheckBox = new JCheckBox("Show Next");
	JCheckBox gridCheckBox = new JCheckBox("Grid");
	JCheckBox soundCheckBox = new JCheckBox("Sound");

	public TertrisDialog(TertrisFrame frame, JFrame parent,
			String titleText,
			String buttonText) {
		super(parent, titleText, true);

		JButton button = new JButton(buttonText);
		JPanel buttonPanel = new JPanel();
		JPanel sliderPanel = new JPanel();
		JPanel togglePanel = new JPanel();

		/*unitsChanger.setWidth(150);
		columnsChanger.setWidth(150);
		fillChanger.setWidth(150);*/
		buttonPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.CENTER));
		sliderPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		sliderPanel.add(unitsChanger, BorderLayout.PAGE_START);
		sliderPanel.add(columnsChanger, BorderLayout.PAGE_START);
		sliderPanel.add(fillChanger, BorderLayout.PAGE_START);
		buttonPanel.add(sliderPanel, BorderLayout.PAGE_START);
		togglePanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		togglePanel.add(fixedFillCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(suddenAppearCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(diagonalCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(mixedCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(bonusCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(clockwiseCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(plainPieceCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(showNextCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(gridCheckBox, BorderLayout.PAGE_START);
		togglePanel.add(soundCheckBox, BorderLayout.PAGE_START);
		buttonPanel.add(togglePanel, BorderLayout.PAGE_START);
		fixedFillCheckBox.setSelected(fixedFillValue);
		suddenAppearCheckBox.setSelected(suddenAppearValue);
		diagonalCheckBox.setSelected(diagonalValue);
		mixedCheckBox.setSelected(mixedValue);
		bonusCheckBox.setSelected(bonusValue);
		clockwiseCheckBox.setSelected(clockwiseValue);
		plainPieceCheckBox.setSelected(plainPieceValue);
		showNextCheckBox.setSelected(showNextValue);
		gridCheckBox.setSelected(gridValue);
		soundCheckBox.setSelected(soundValue);
		buttonPanel.add(button, BorderLayout.PAGE_START);
		add(BorderLayout.PAGE_START, buttonPanel);
		pack();
		TertrisDialog.active = true;
		TertrisDialog.frame = frame;
		unitsChanger.addChangeListener(new UnitsChangeListener());
		columnsChanger.addChangeListener(new ColumnsChangeListener());
		fillChanger.addChangeListener(new FillChangeListener());
		fixedFillCheckBox.addItemListener(new FixedFillListener());
		suddenAppearCheckBox.addItemListener(new
			SuddenAppearListener());
		diagonalCheckBox.addItemListener(new DiagonalListener());
		mixedCheckBox.addItemListener(new MixedListener());
		bonusCheckBox.addItemListener(new BonusListener());
		clockwiseCheckBox.addItemListener(new ClockwiseListener());
		plainPieceCheckBox.addItemListener(new PlainPieceListener());
		showNextCheckBox.addItemListener(new ShowNextListener());
		gridCheckBox.addItemListener(new GridListener());
		soundCheckBox.addItemListener(new ToggleSoundListener());

		button.addActionListener((ActionEvent event) -> {
			dispose();
			//else multiple Dialogs can be opened
			active = false;
		});

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				dispose();
				//else multiple Dialogs can be opened
				active = false;
			}
		});
	}

	class UnitsChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = unitsValue;

			unitsValue = unitsChanger.getValue();
			if (oldValue != unitsValue) {
				frame.setUnits(unitsValue);
				frame.refreshNext();
			}
		}
	}

	class ColumnsChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = columnsValue;

			columnsValue = columnsChanger.getValue();
			if (oldValue != columnsValue) {
				frame.endGame();
				frame.setColumns(columnsValue);
				frame.startGame();
			}
		}
	}

	class FillChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = fillValue;

			fillValue = fillChanger.getValue();
			if (oldValue != fillValue) {
				frame.endGame();
				frame.setFill(fillValue);
				frame.startGame();
			}
		}
	}

	class FixedFillListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setFixedFill(fixedFillCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class SuddenAppearListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setSuddenAppear(
				suddenAppearCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class DiagonalListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setDiagonal(diagonalCheckBox.isSelected());
			frame.refreshNext();
			frame.requestFocus();
		}
	}

	class MixedListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setMixed(mixedCheckBox.isSelected());
			frame.refreshNext();
			frame.requestFocus();
		}
	}

	class BonusListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setBonus(bonusCheckBox.isSelected());
			frame.refreshNext();
			frame.requestFocus();
		}
	}

	class ClockwiseListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setClockwise(clockwiseCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class PlainPieceListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setPlainPiece(plainPieceCheckBox.isSelected());
			frame.changeShowNext();
			frame.requestFocus();
		}
	}

	class ShowNextListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			showNextValue = showNextCheckBox.isSelected();
			frame.setShowNext(showNextValue);
			frame.requestFocus();
		}
	}

	class GridListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setGrid(gridCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class ToggleSoundListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.toggleSound();
			frame.requestFocus();
		}
	}

	public synchronized void setUnits(int unitsValue) {
		TertrisDialog.unitsValue = unitsValue;
		unitsChanger.setValue(unitsValue);
	}

	public static synchronized int getUnits() {
		return unitsValue;
	}

	public void setColumns(int columnsValue) {
		TertrisDialog.columnsValue = columnsValue;
		columnsChanger.setValue(columnsValue);
	}

	public static int getColumns() {
		return columnsValue;
	}

	public void setFill(int fillValue) {
		TertrisDialog.fillValue = fillValue;
		fillChanger.setValue(fillValue);
	}

	public static int getFill() {
		return fillValue;
	}

	public void setFixedFill(boolean fixedFillValue) {
		TertrisDialog.fixedFillValue = fixedFillValue;
		fixedFillCheckBox.setSelected(fixedFillValue);
	}

	public static boolean getFixedFill() {
		return fixedFillValue;
	}

	public void setSuddenAppear(boolean suddenAppearValue) {
		TertrisDialog.suddenAppearValue = suddenAppearValue;
		suddenAppearCheckBox.setSelected(suddenAppearValue);
	}

	public static boolean getSuddenAppear() {
		return suddenAppearValue;
	}

	synchronized void setDiagonal(boolean diagonalValue) {
		TertrisDialog.diagonalValue = diagonalValue;
		diagonalCheckBox.setSelected(diagonalValue);
	}

	public static synchronized boolean getDiagonal() {
		return diagonalValue;
	}

	public synchronized void setMixed(boolean mixedValue) {
		TertrisDialog.mixedValue = mixedValue;
		mixedCheckBox.setSelected(mixedValue);
	}

	public static synchronized boolean getMixed() {
		return mixedValue;
	}

	public void setBonus(boolean bonusValue) {
		TertrisDialog.bonusValue = bonusValue;
		bonusCheckBox.setSelected(bonusValue);
	}

	public static boolean getBonus() {
		return bonusValue;
	}

	public void setClockwise(boolean clockwiseValue) {
		TertrisDialog.clockwiseValue = clockwiseValue;
		clockwiseCheckBox.setSelected(clockwiseValue);
	}

	public static boolean getClockwise() {
		return clockwiseValue;
	}

	public void setPlainPiece(boolean plainPieceValue) {
		TertrisDialog.plainPieceValue = plainPieceValue;
		plainPieceCheckBox.setSelected(plainPieceValue);
	}

	public static boolean getPlainPiece() {
		return plainPieceValue;
	}

	public void setShowNext(boolean showNextValue) {
		TertrisDialog.showNextValue = showNextValue;
		showNextCheckBox.setSelected(showNextValue);
	}

	public static boolean getShowNext() {
		return showNextValue;
	}

	public void setGrid(boolean gridValue) {
		TertrisDialog.gridValue = gridValue;
		gridCheckBox.setSelected(gridValue);
	}

	public static boolean getGrid() {
		return gridValue;
	}

	public void setToggleSound(boolean soundValue) {
		TertrisDialog.soundValue = soundValue;
		soundCheckBox.setSelected(soundValue);
	}

	public static boolean getToggleSound() {
		return soundValue;
	}

	public static boolean getActive() {
		return TertrisDialog.active;
	}
}
