package com.sillycycle.bagleyd.tertris;

/*
 * @(#)TertrisInterface.java
 *
 * Copyright 1993 - 2019  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * The <code>TertrisInterface</code> class contains most of the
 * constants used by the <code>TertrisFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class TertrisInterface {
	static final String XML_FILE = "polyiamond.xml";

	static final String SCOREFILE = "tertris.score";
	static final int NUM_FLASHES = 4;

	/* static final int BONUSIN = 25000;
	static final int BONUSOUT = 1000;
	static final int SAMPLERATE = 5000000; */

	static final int MAX_SCORES = 3;
	static final int SHOWSCORES = 15;

	static final int EMPTY = -1;
	static final int NEXTTRIWIDTH = 36;
	static final int NEXTTRIHEIGHT = 29; /* TRIWIDTH*3^(1/2)/2 */
	static final int MIN_LEVELS = 1;
	static final int MAX_LEVELS = 20;
	static final int MAX_SIDES = 6;
	static final int DEFAULT_COLUMNS = 15; /* odd */
	static final int DEFAULT_ROWS = 20;
	static final int MIN_COLUMNS = 9; /* odd */
	static final int MAX_COLUMNS = 29; /* odd */
	static final int DEFAULT_FILL = 4;
	static final int MAX_FILL = DEFAULT_COLUMNS;
	static final int DEFAULT_LEVEL = 3;

	static final String MSG_PAUSED = "Paused";
	static final String MSG_END = "Game Over";
	//static final String TITLE = "Tertris";
	static final String TITLE = "Tert\u044fis";
	public enum Icons {X16, X22, X24, X32, X48, X64}
	static final String[] ICONS = {"icons/16x16/tertris.png",
		"icons/22x22/tertris.png",
		"icons/24x24/tertris.png",
		"icons/32x32/tertris.png",
		"icons/48x48/tertris.png",
		"icons/64x64/tertris.png"};
	public enum Sounds {BUMP}
	static final String[] SOUNDS = {"sounds/bump.wav"};

	static boolean bonusLegal(int units, boolean bonus) {
		return (units <= PolyiamondThing.MAX_UNITS - ((bonus) ? 1 : 0));
	}
	static boolean mixedLegal(int units, boolean mixed) {
		return (units >= PolyiamondThing.MIN_UNITS + ((mixed) ? 1 : 0));
	}
	static synchronized int threshold(int x) {
		return ((x + 1) * 10 + x);
	}

	static final int FALL = 1, DROP = 2, MOVE_LEFT = 3, MOVE_RIGHT = 4,
		ROTATE = 5, REFLECT = 6;
}
